import { InjectionToken, Injectable, Optional, Inject, Component, HostBinding, Input, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import prismjs from 'prismjs';
import { of, from } from 'rxjs';
import { map } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/** Prism Language Modules Token */
const LANGUAGE_MODULES = new InjectionToken('wizdm.prism.language.modules');
/** @dynamic - tells ngc to ignore the error on type PrismLanguages generated by strictEmitMetadata: true */
class PrismService {
    constructor(languages) {
        this.languages = languages;
    }
    /** Tokenizes the input source by loading the grammar dynamically whenever necessary */
    tokenize(source, language) {
        // Skips invalid source
        if (!source) {
            return of(['']);
        }
        // Loads the grammar and tokenizes the source accordingly
        return this.loadGrammar(language).pipe(map(grammar => !!grammar ? prismjs.tokenize(source, grammar) : [source]));
    }
    /** Loads the Grammar */
    loadGrammar(language) {
        // No language, no grammar...
        if (!language) {
            return of(undefined);
        }
        // Verifies the grammar has been already loaded
        if (prismjs.languages[language]) {
            return of(prismjs.languages[language]);
        }
        // Attempts to load the grammar dynamically
        const loader = this.languages && this.languages.find(({ name }) => {
            // Matches the requested language
            return Array.isArray(name) ? (name.indexOf(language) >= 0) : (name === language || name === '*');
        });
        // Abort when no loader is found
        if (!loader) {
            return of(undefined);
        }
        // Resolves the grammar by loading the module
        return from(loader.load(language)).pipe(map(() => prismjs.languages[language]));
    }
}
PrismService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
PrismService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_MODULES,] }] }
];

/** Perform code hilighting by processing an input text to be rendered into an angular template
 * Using prism as tokenizer @see {https://github.com/PrismJS/prism} */
class PrismHighlighter {
    constructor(prism) {
        this.prism = prism;
        this.disabled = false;
    }
    /** Applies the proper classes to the host <pre> element */
    get clazz() {
        return `wm-prism${this.disabled ? '' : ' language-none'}`;
    }
    /** Disables the highlighting */
    set disableHighlight(value) { this.disabled = coerceBooleanProperty(value); }
    /** Parses the source text */
    set source(source) {
        this.tokens$ = this.prism.tokenize(source, this.language);
    }
}
PrismHighlighter.decorators = [
    { type: Component, args: [{
                selector: 'pre[wm-prism]',
                template: "\n<code *ngIf=\"!disabled; else transclude\" [tokenize]=\"tokens$ | async\"></code>\n\n<ng-template #transclude>\n  <ng-content></ng-content>\n</ng-template>"
            },] }
];
/** @nocollapse */
PrismHighlighter.ctorParameters = () => [
    { type: PrismService }
];
PrismHighlighter.propDecorators = {
    clazz: [{ type: HostBinding, args: ['class',] }],
    disableHighlight: [{ type: Input, args: ['disabled',] }],
    language: [{ type: Input }],
    source: [{ type: Input, args: ['wm-prism',] }]
};

// Forces prism to run in manual mode
(function (window) {
    window.Prism = (window.Prism || {});
    window.Prism.manual = true;
}(window || {}));

class PrismTokenizer {
    /** Tokenizes the input string or pass along the already tokenized array */
    set tokenize(source) {
        this.tokens = typeof (source) === 'string' ? [source] : source;
    }
    /** Helper for rendering strings */
    isString(token) { return typeof (token) === 'string'; }
    /** Helper for rendering tokens */
    tokenClass(token) {
        // Returns the basic token class + type and appends the aliases, if any 
        return token ? ('token ' + (token.type || '') + this.tokenAliases(token)) : '';
    }
    tokenAliases(token) {
        // Skips when no aliases
        if (!token.alias) {
            return '';
        }
        // Appends the multiple aliases
        if (token.alias instanceof Array) {
            return token.alias.reduce((c, alias) => c + ' ' + alias, '');
        }
        // Appends the single alias
        return ' ' + token.alias;
    }
}
PrismTokenizer.decorators = [
    { type: Component, args: [{
                selector: '[tokenize]',
                template: "<!-- Loops on tokens -->\n<ng-container *ngFor=\"let token of tokens\">\n\n  <!-- Renders plain strings as text nodes -->\n  <ng-container *ngIf=\"isString(token); else spanToken\">{{ token }}</ng-container>\n\n  <!-- Spans the token recurring down the tree with the proper styling otherwise -->\n  <ng-template #spanToken>\n    <span [class]=\"tokenClass(token)\" [tokenize]=\"token.content\"></span>\n  </ng-template>\n\n</ng-container>"
            },] }
];
PrismTokenizer.propDecorators = {
    tokenize: [{ type: Input }]
};

class PrismModule {
    /** Initializes the PrismModule with extra language loaders */
    static init(languages) {
        return {
            ngModule: PrismModule,
            providers: [{ provide: LANGUAGE_MODULES, useValue: languages }]
        };
    }
}
PrismModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    PrismHighlighter,
                    PrismTokenizer
                ],
                imports: [CommonModule],
                exports: [PrismHighlighter],
                providers: [PrismService]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { LANGUAGE_MODULES, PrismHighlighter, PrismModule, PrismService, PrismTokenizer as ɵa };
//# sourceMappingURL=wizdm-prism.js.map
